
#include <qapplication.h>
#include <qtranslator.h>
#include <qtextcodec.h>
#include "kphone.h"

#include <qsqldatabase.h>
#include <unistd.h>
#include <qsettings.h>
#include <qmessagebox.h>

#include <qsplashscreen.h>
#include <qpixmap.h>
#include <qtimer.h>

QString client;
QString server;
QString realm;
QString display;
QString number;
QString username;
QString dbpasswd;
int usestun;
QString stunserver;
QString countryprefix;
QString agentKey;
QString contractKey;
QString callnumkey;
QString phonekey;
int statusBuffer;
int dirtyBuffer;
int pauseStarted = 0;
QString agentNormal;
int contractFree;
double microVol;
double speakerVol;
int echoCancel = 0;
int originalUid;

bool createConnections()
{
	// create the default database connection
	QSqlDatabase *defaultDB = QSqlDatabase::addDatabase( "QMYSQL3" );
	if ( ! defaultDB ) {
		QMessageBox::critical( 0, QObject::tr( "Startup failed" ), QObject::tr( "Database driver could not be loaded" ) );
		qWarning( "Failed to connect to driver" );
		return FALSE;
	}
	QSettings settings;
	defaultDB->setDatabaseName( settings.readEntry( "/kallcenter/mysql/DatabaseName", "callcenter" ) );
	defaultDB->setUserName( settings.readEntry( "/kallcenter/mysql/UserName", "callcenter" ) );
	defaultDB->setPassword( settings.readEntry( "/kallcenter/mysql/Password", "callcenter" ) );
	defaultDB->setHostName( settings.readEntry( "/kallcenter/mysql/HostName", "localhost" ) );
	if ( ! defaultDB->open() ) {
		QMessageBox::critical( 0, QObject::tr( "Startup failed" ), QObject::tr( "Database could not be opened" ) );
		qWarning( "Failed to open callcenter database: " +
			  defaultDB->lastError().driverText() );
		qWarning( defaultDB->lastError().databaseText() );
		return FALSE;
	}

	char clientname[100];
	if ( gethostname(clientname, 100) != 0 )
		return FALSE;
	client = clientname;

	QSqlQuery q ( "select server, realm, display, number, username, password, usestun, stunserver, countryprefix, pkey from sip where client = '" + client + "';" );
	if ( q.next() ) {
		server		= q.value( 0 ).toString();
		realm		= q.value( 1 ).toString();
		display		= q.value( 2 ).toString();
		number		= q.value( 3 ).toString();
		username	= q.value( 4 ).toString();
		dbpasswd	= q.value( 5 ).toString();
		usestun		= q.value( 6 ).toInt();
		stunserver	= q.value( 7 ).toString();
		countryprefix	= q.value( 8 ).toString();
		phonekey	= q.value( 9 ).toString();
	}
	else {
		QMessageBox::critical( 0, QObject::tr( "Startup failed" ), QObject::tr( "No SIPPhone defined for this host" ) );
		qWarning( "No SIPPhone defined for host " + client );
		return FALSE;
	}

	return TRUE;
}

int main( int argc, char **argv )
{
	originalUid = getuid();
	setuid( 0 );

	int listenport = 5060;
	QString port = "";
	QString user = "";
	bool looseRoute = false;
	bool strictRoute = false;

	printf( "started at %s\n", QDateTime::currentDateTime().toString( "dd.MM.yyyy hh:mm:ss" ).ascii() );
	char *stdoutbuf = 0;
	printf( "stdout buffer: %d\n", setvbuf(stdout, stdoutbuf, _IONBF, 0) );

	// kill all potentially sound blocking applications
	system( "/usr/bin/killall artsd" );
	system( "/bin/fuser -k /dev/dsp* /dev/snd/*" );
	// restore originally sound settings
	system( "if [ ! -f $HOME/.qt/kcenter.alsa ]; then /usr/sbin/alsactl -f $HOME/.qt/kcenter.alsa store; fi" );
	system( "/usr/sbin/alsactl -f $HOME/.qt/kcenter.alsa restore" );

	printf( "sound system ready\n" );

	QSettings settings;
	echoCancel = settings.readEntry( "/kallcenter/sound/echoCancel", "0" ).toInt();
	microVol = settings.readEntry( "/kallcenter/sound/microVol", "2.0" ).toDouble();
	speakerVol = settings.readEntry( "/kallcenter/sound/speakerVol", "1.0" ).toDouble();

	QApplication a( argc, argv );

	// translation file for application strings
	setreuid( originalUid, -1 ); // otherwise LANG is taken from root
	QTranslator myapp( 0 );
	myapp.load( QString( "kallcenter_" ) + QTextCodec::locale(), PO_DIR );
	a.installTranslator( &myapp );
	setreuid( 0, -1 );
	QFont f( "sansserif", 11 );
	a.setFont( f );

	printf( "kallcenter Version: %s %s\n", __DATE__, __TIME__);
	QPixmap pixmap( SHARE_DIR "/icons/splash.png" );
	QSplashScreen *splash = new QSplashScreen( pixmap );
	splash->show();
	splash->message( "Version: " __DATE__ " " __TIME__ );
	QTimer *timer = new QTimer( splash );
	QTimer::connect( timer, SIGNAL(timeout()), splash, SLOT( close() ) );
	timer->start( 5000, TRUE ); // 2 seconds single-shot timer

	if ( ! createConnections() )
		return 1;

	for ( int i = 0; i < a.argc(); i++ ) {
		if( QString( a.argv()[i] ) == "-u" ) {
			if( i+1 < a.argc() ) {
				user = QString( a.argv()[++i] );
			}
		} else if( QString( a.argv()[i] ) == "-p" ) {
			if( i+1 < a.argc() ) {
				port = QString( a.argv()[++i] );
			}
			listenport = port.toInt();
		} else if( QString( a.argv()[i] ) == "-l" ) {
			looseRoute = true;
		} else if( QString( a.argv()[i] ) == "-s" ) {
			strictRoute = true;
		}
	}
	KPhone *phone = new KPhone( listenport, user, looseRoute, strictRoute );
	a.setMainWidget( phone );
	phone->show();
	return a.exec();
}
